@@ -34,6 +34,9 @@ import org.jruby.util.TypeConverter;
  * @author mernen
  */
 public class GeneratorState extends RubyObject {
+    private boolean allowDuplicateKey = false;
+    private boolean deprecateDuplicateKey = true;
+
     /**
      * The indenting unit string. Will be repeated several times for larger
      * indenting levels.
@@ -82,12 +85,6 @@ public class GeneratorState extends RubyObject {
      */
     private boolean asciiOnly = DEFAULT_ASCII_ONLY;
     static final boolean DEFAULT_ASCII_ONLY = false;
-    /**
-     * If set to <code>true</code> all JSON values generated might not be
-     * RFC-conform JSON documents.
-     */
-    private boolean quirksMode = DEFAULT_QUIRKS_MODE;
-    static final boolean DEFAULT_QUIRKS_MODE = false;
     /**
      * If set to <code>true</code> the forward slash will be escaped in
      * json output.
@@ -218,11 +215,14 @@ public class GeneratorState extends RubyObject {
         this.maxNesting = orig.maxNesting;
         this.allowNaN = orig.allowNaN;
         this.asciiOnly = orig.asciiOnly;
-        this.quirksMode = orig.quirksMode;
         this.scriptSafe = orig.scriptSafe;
         this.strict = orig.strict;
         this.bufferInitialLength = orig.bufferInitialLength;
         this.depth = orig.depth;
+
+        this.allowDuplicateKey = orig.allowDuplicateKey;
+        this.deprecateDuplicateKey = orig.deprecateDuplicateKey;
+
         return this;
     }
 
@@ -486,6 +486,24 @@ public class GeneratorState extends RubyObject {
         return str.getByteList().dup();
     }
 
+    @JRubyMethod(name="allow_duplicate_key?", visibility=Visibility.PRIVATE)
+    public IRubyObject allow_duplicate_key_p(ThreadContext context) {
+        if (allowDuplicateKey) {
+            return context.runtime.getTrue();
+        } else if (deprecateDuplicateKey) {
+            return context.runtime.getNil();
+        }
+        return context.runtime.getFalse();
+    }
+
+    public boolean getAllowDuplicateKey() {
+        return allowDuplicateKey;
+    }
+
+    public boolean getDeprecateDuplicateKey() {
+        return deprecateDuplicateKey;
+    }
+
     /**
      * <code>State#configure(opts)</code>
      *
@@ -527,6 +545,10 @@ public class GeneratorState extends RubyObject {
 
         depth = opts.getInt("depth", 0);
 
+        if (opts.hasKey("allow_duplicate_key")) {
+            this.allowDuplicateKey = opts.getBool("allow_duplicate_key", false);
+            this.deprecateDuplicateKey = false;
+        }
         return this;
     }
 
@@ -555,6 +577,16 @@ public class GeneratorState extends RubyObject {
         result.op_aset(context, runtime.newSymbol("strict"), strict_get(context));
         result.op_aset(context, runtime.newSymbol("depth"), depth_get(context));
         result.op_aset(context, runtime.newSymbol("buffer_initial_length"), buffer_initial_length_get(context));
+
+        if (this.allowDuplicateKey) {
+            if (!this.deprecateDuplicateKey) {
+                result.op_aset(context, runtime.newSymbol("allow_duplicate_key"), runtime.getTrue());
+            }
+        }
+        else {
+            result.op_aset(context, runtime.newSymbol("allow_duplicate_key"), runtime.getFalse());
+        }
+
         for (String name: getInstanceVariableNameList()) {
             result.op_aset(context, runtime.newSymbol(name.substring(1)), getInstanceVariables().getInstanceVariable(name));
         }
